/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief
 *      Headerfile for battery.c
 *
 *      Contains definitions for the setup of 1-Wire(R) and behaviour of the
 *      battery data and status refreshing functions.\n
 *      Also contains definitions of default battery data, and declarations
 *      for the RID and NTC lookup tables.
 *
 * \par Application note:
 *      AVR458: Charging Li-Ion Batteries with BC100
 *
 * \par Documentation:
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com \n
 *      Original author: \n
 *
 * $Name$
 * $Revision: 2261 $
 * $RCSfile$
 * $URL: http://svn.norway.atmel.com/AppsAVR8/avr458_Charging_Li-Ion_Batteries_with_BC100/tag/20070904_release_1.0/code/IAR/battery.h $
 * $Date: 2007-08-10 09:28:35 +0200 (fr, 10 aug 2007) $\n
 ******************************************************************************/


#ifndef BATTERY_H
#define BATTERY_H

#include <avr/eeprom.h>


//******************************************************************************
// Defines for EEPROM reading
//******************************************************************************
#define OWIBUS  OWI_PIN_0         //!< One-wire bus (pin).

#define OW_NONE           0x00    //!< One-wire device family code.
#define OW_DS2505         0x09    //!< One-wire device family code.

#define DS2505_MEM_READ     0xF0  //!< One-wire device command.
#define DS2505_STATUS_READ  0xAA  //!< One-wire device command.
#define DS2505_DATA_READ    0xC3  //!< One-wire device command.


//******************************************************************************
// RID-less charging (for BatteryStatusRefresh())
//******************************************************************************

#ifdef EASYPACK550

#warning "using EasyPack 66590 711 099 (550mAh) settings - mthomas, no warranty!"
// RID not connected
#define ALLOW_NO_RID                //!< Use default battery data if no matching entry found.
#define DEF_BAT_CAPACITY       550  //!< Default battery capacity, in mAh.
#define DEF_BAT_CURRENT_MAX    260  //!< Default maximum charge current, in mA.
#define DEF_BAT_TIME_MAX    (5*60)  //!< Default maximum charge time, in minutes.
//! Default minimum current to stop charge, in mA.
#define DEF_BAT_CURRENT_MIN     10

#else

//#define ALLOW_NO_RID  //!< Use default battery data if no matching entry found.

#define DEF_BAT_CAPACITY		0  //!< Default battery capacity, in mAh.
#define DEF_BAT_CURRENT_MAX	0  //!< Default maximum charge current, in mA.
#define DEF_BAT_TIME_MAX		0  //!< Default maximum charge time, in minutes.
//! Default minimum current to stop charge, in mA.
#define DEF_BAT_CURRENT_MIN	0

#endif /* EASYPACK550 */


//******************************************************************************
// RID and NTC defines and struct declarations
//******************************************************************************
#define RID_TABLE_SIZE        4     //!< Number of entries in RID table.
#define NTC_TABLE_SIZE        20    //!< Number of entries in NTC table.

//! \brief Struct for an entry in the resistor ID lookup-table
struct RID_Lookup_struct {
  unsigned int  Low;              //!< Lowest acceptable ADC value.
  unsigned int  High;             //!< Highest acceptable ADC value.
  unsigned int  Resistance;       //!< RID resistance, in Ohms.
  unsigned int  Capacity;         //!< Associated battery capacity, in mAh.
  unsigned int  Icharge;          //!< Associated initial charge current, in mA.
  unsigned int  tCutOff;          //!< Associated cut-off time, in minutes.
  unsigned int  ICutOff;          //!< Associated cut-off current, in mA.
};
typedef struct RID_Lookup_struct RID_Lookup_t; //!< For convenience.

/*! \brief Struct for an entry in the NTC lookup-table
 *
 * \note Must be sorted in descending ADC order.
 */
struct NTC_Lookup_struct
{
  unsigned int  ADCV;  //!< Measured NTC.
  unsigned char ADCsteps;  //!< ADC steps per half degree at this temperature.
};
typedef struct NTC_Lookup_struct NTC_Lookup_t; //!< For convenience.


//******************************************************************************
// Global variables
//******************************************************************************
extern Battery_t EEMEM BattControl[];
extern Batteries_t BattData;
extern unsigned char EEMEM BattEEPROM[][32];
extern unsigned char BattActive;


//******************************************************************************
// Function prototypes
//******************************************************************************
unsigned char BatteryCheck(void);
unsigned char BatteryStatusRefresh(void);
unsigned char BatteryDataRefresh(void);
void EnableBattery(unsigned char);
void DisableBatteries(void);
unsigned char RIDLookUp(void);
void NTCLookUp(void);


#endif // BATTERY_H
