/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief
 *      Functions for timing
 *
 *      Contains functions to initialize, set, poll and stop timers.
 *
 * \par Application note:
 *      AVR458: Charging Li-Ion Batteries with BC100 \n
 *      AVR463: Charging NiMH Batteries with BC100
 *
 * \par Documentation
 *      For comprehensive code documentation, supported compilers, compiler 
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com
 *
 * 
 * $Name$
 * $Revision: 2299 $
 * $RCSfile$
 * $URL: http://svn.norway.atmel.com/AppsAVR8/avr458_Charging_Li-Ion_Batteries_with_BC100/tag/20070904_release_1.0/code/IAR/time.c $
 * $Date: 2007-08-23 12:55:51 +0200 (to, 23 aug 2007) $\n
 ******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>

#include "enums.h"

#include "main.h"
#include "time.h"


//******************************************************************************
// Variables
//******************************************************************************
volatile unsigned long timeval[TIMERS];  //!< Contains the values for each timer.

// timer runs at 1 MHz and overflow will occur every 255 / 1 Mz ~= 0.25 ms 
//#pragma vector = TIM0_OVF_vect


//******************************************************************************
// Functions
//******************************************************************************
/*! \brief Interrupt service routine for timer 0 overflow
 *
 * Timer 0 runs at 125 kHz and compare match will occur every millisecond
 * (125 / 125 kHz = 1.0 ms), which will result in a call to this function.
 * When called, this function will decrement the time left for each timer,
 * unless they are already at zero.
 */
ISR(TIMER0_COMPA_vect)
{
	unsigned char i;

	// 1 ms has passed, decrement all non-zero timers.
	for (i = 0; i < TIMERS; i++) {
		if(timeval[i] > 0) {
			timeval[i]--;
		}
	}
}


/*! \brief Checks if a specified timer has expired
 *
 * \param timer Specifies timer
 *
 * \retval TRUE Timer still going.
 * \retval FALSE Timer has expired.
 */ 
unsigned char Time_Left(unsigned char timer)
{
	unsigned long tval;
	unsigned char sreg_saved;

	sreg_saved = SREG;
	cli();
	tval = timeval[timer];
	SREG = sreg_saved;

	if( tval > 0 ) {
		return(TRUE);
	} else {
		return(FALSE);
	}
}


/*! \brief Sets the specified timer
 *
 * \param timer Specifies timer
 * \param min Minutes for timer to count down
 * \param sec Seconds for timer to count down
 * \param ms Milliseconds for timer to count down
 */
void Time_Set(unsigned char timer, unsigned int min, unsigned char sec,
				  unsigned char ms)
{
//	timeval[i] = 4 * (1000*(sec + 60*min) + ms);   // about 4000 ticks per second
//	timeval[i] = 240000 * (unsigned long)min;
//	timeval[i] += 4000 * (unsigned long)sec;
//	timeval[i] += 4 * (unsigned long)ms;

	unsigned long tval;
	unsigned char sreg_saved;
	

	tval  = 60000 * (unsigned long)min;
	tval += 1000 * (unsigned long)sec;
	tval += 1 * (unsigned long)ms;

	sreg_saved = SREG;
	cli();
	timeval[timer] = tval;
	SREG = sreg_saved;
}


/*! \brief Stops timers
 *
 * Sets timer0's clock source to none.
 */
void Time_Stop(void)
{
	TCCR0B = 0;
}


/*! \brief Starts timers
 *
 * Sets timer0's clock source to system clock divided by 64.
 */
void Time_Start(void)
{
	TCCR0B = (0<<CS02)|(1<<CS01)|(1<<CS00);  	// CLKT0 = CLK/64 = 125 kHz.
}


/*! \brief Initializes timers
 *
 * Resets all the timer values to 0, then sets up timer 0 for a compare match
 * every millisecond.
 */
void Time_Init(void)
{
	unsigned char i;
	unsigned char sreg_saved;
	
	sreg_saved = SREG;
	cli();
	for (i = 0; i<<TIMERS; i++)	{
		timeval[i] = 0;
	}
	SREG = sreg_saved;

	//    OCR0A = 0;  // Doesn't matter, will run in normal mode.
	
	OCR0A = 125;  // Will give a compare match every ms.
	
	OCR0B = 0;  // Doesn't matter, will run in normal mode.

		//    TCCR0A = 0;  // Normal 8-bit mode, no input capture.

	TCCR0A = (1<<WGM00);  // 8-bit CTC mode.
	
	//    TCCR0B = (0<<CS02)|(1<<CS01)|(0<<CS00);  // CLKT0 = CLK/8 = 1 MHz.
	
	TCCR0B = (0<<CS02)|(1<<CS01)|(1<<CS00);  	// CLKT0 = CLK/64 = 125 kHz.

	//    TIMSK |= (1<<TOIE0);	// Overflow interrupt enabled.

	TIMSK |= (1<<OCIE0A);  // Timer 0, Compare match A interrupt enabled.
}
