/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief
 *      State functions
 *
 *      Contains the functions related to the states defined in menu.h.\n
 *      Also contains related functions, i.e. for checking jumpers, setting
 *      error flags and "dozing".
 *
 *      \note The state function Charge() is in a separate file since it
 *      should easily be changed with regard to battery type.
 *
 * \par Application note:
 *      AVR458: Charging Li-Ion Batteries with BC100 \n
 *      AVR463: Charging NiMH Batteries with BC100
 *
 * \par Documentation
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com
 *
 *
 * $Name$
 * $Revision: 2299 $
 * $RCSfile$
 * $URL: http://svn.norway.atmel.com/AppsAVR8/avr458_Charging_Li-Ion_Batteries_with_BC100/tag/20070904_release_1.0/code/IAR/statefunc.c $
 * $Date: 2007-08-23 12:55:51 +0200 (to, 23 aug 2007) $\n
 ******************************************************************************/

#include <ioavr.h>
#include <inavr.h>
#include <stdlib.h>

#include "structs.h"
#include "enums.h"

#include "ADC.h"
#include "statefunc.h"
#include "battery.h"
#include "charge.h"
#include "main.h"
#include "menu.h"
#include "OWI.h"
#include "PWM.h"
#include "time.h"
#include "USI.h"


//******************************************************************************
// Variables
//******************************************************************************
unsigned char ErrorFlags;  //!< \brief Holds error flags.
                           //!< \note See statefunc.h for definitions of flags.

//! \brief Holds the state in which latest error flag was set.
//! \note See menu.h for definitions of states.
unsigned char ErrorState;

//! \brief Set to 1 by the watchdog-timeout-ISR
//! \note added by Martin Thomas
volatile unsigned char WatchdogFlag;

//******************************************************************************
// Functions
//******************************************************************************
/*! \brief Initialization
 *
 * - Sets the system clock prescaler to 1 (run at 8 MHz)
 * - Initializes the one-wire interface
 * - Clears on-chip EEPROM
 * - Sets battery enable pins as outputs, then disables batteries
 * - Initializes SPI according to \ref SPIMODE
 * - Initializes ADC
 * - Initializes timers
 * - Reads battery data from both battery inputs (via ADC)
 * - Disables batteries again
 * - Sets battery A as the current one (\ref BattActive = 0)
 * - Clears ErrorFlags
 *
 * \param inp Not used.
 *
 * \retval ST_BATCON Next state in the sequence.
 */
unsigned char Initialize(unsigned char inp)
{
	unsigned char i, page;

	// Disable interrupts while setting prescaler.
	__disable_interrupt();

	CLKPR = (1<<CLKPCE);          // Enable CLKPS bit modification.
	CLKPR = 0;                    // Set prescaler 1 => 8 MHz clock frequency.

	// Init 1-Wire(R) interface.
	OWI_Init(OWIBUS);

	// Clear on-chip EEPROM.
	for (page = 0; page < 4; page++) {
		for (i = 0; i < 32; i++) {
			// From mthomas GCC addition
			if (BattEEPROM[page][i] != 0) {
				BattEEPROM[page][i] = 0;
			}
		}
	}

	DDRB = (1<<PB4) | (1<<PB5);   // Set battery enable pins as outputs.
	DisableBatteries();
	SPI_Init(SPIMODE);
	ADC_Init();
	Time_Init();

	// Attempt to get ADC-readings (also gets RID-data) from both batteries.
	for (i = 0; i < BATCONN; i++)	{
		EnableBattery(i);
		ADC_Wait();
		BatteryStatusRefresh();
	}

	DisableBatteries();

	BattActive = 0;               // We have to start somewhere..
	ErrorFlags = 0;

	// Init complete! Go to ST_BATCON next.
	return(ST_BATCON);
}


/*! \brief Tests jumper settings and batteries, starts charging if necessary.
 *
 * First, JumperCheck() is called. If successful, the function checks if any
 * valid batteries are connected and attempts to charge these, if necessary.\n
 * If no charging is necessary, the charger goes to ST_SLEEP next.\n
 * ST_ERROR is next if either JumperCheck() fails or there are no valid
 * batteries. In this last case, the error is also flagged.
 *
 * \param inp Not used.
 *
 * \retval ST_ERROR Next state if either the jumper check failed, or there are
 * no valid batteries.
 * \retval ST_PREQUAL Next state if a battery is found to enabled and not fully
 * charged.
 * \retval ST_SLEEP Next state if battery/batteries are enabled and fully
 * charged.
 */
unsigned char BatteryControl(unsigned char inp)
{
	unsigned char i;

	// Make sure ADC inputs are configured properly! (Will disables batteries.)
	if (!JumperCheck()) {
		return(ST_ERROR);           // Error. Exit before damage is done!
	}

	// If neither battery is valid, flag error and go to error state
	if ((!BattControl[0].Enabled) && (!BattControl[1].Enabled)) {
		SetErrorFlag(ERR_NO_BATTERIES_ENABLED);

		return(ST_ERROR);
	}

	// Get ADC-readings, try to read EPROM, and start prequalification
	// of any uncharged battery.
	for (i = 0; i < BATCONN; i++) {
		if (BattControl[i].Enabled) {
			EnableBattery(i);
			ADC_Wait();

			if (BatteryStatusRefresh()) {
				if (!BattData.Charged) {
					BatteryDataRefresh();

					return(ST_PREQUAL);
				}
			}
		}
	}

	// If we end up here, one or two batteries are found and fully charged.
	// Disconnect, so we don't drain them, and go to sleep.
	DisableBatteries();

	return(ST_SLEEP);
}


/*! \brief Start running on batteries
 *
 * \todo Run on batteries, if battery voltage high enough.
 * \todo Jump here when mains voltage drops below threshold
 *
 */
unsigned char Discharge(unsigned char inp)
{
	return(ST_BATCON);  // Supply voltage restored, start charging
}


/*! \brief Sleeps until either battery needs charging
 *
 * Calls Doze(), then refreshes the status for both batteries on wakeup. If
 * connected batteries are both charged, the function will loop. If not, it's
 * back to ST_BATCON.
 *
 * \param inp Not used.
 *
 * \retval ST_BATCON Next state if a connected battery isn't fully charged.
 */
unsigned char Sleep(unsigned char inp)
{
	unsigned char i;

	do {
		Doze();               // Take a nap (~8 seconds).

		// If any batteries need charging, go to ST_BATCON.
		// Otherwise, keep sleeping.
		for (i = 0; i < BATCONN; i++) {
			EnableBattery(i);
			ADC_Wait();
			if ( BatteryStatusRefresh() ) {
				if ( !BattData.Charged ) {
                                  return(ST_BATCON);
                                }
			}
		}

		DisableBatteries();  // Disable both batteries before Doze()!
	} while (TRUE);
}

/*! \brief Watchdog interrupt-service-routine
 *
 * Called on watchdog timeout, added by Martin Thomas
 */
#pragma vector = WDT_vect
__interrupt void WDT_ISR(void)
{
	WatchdogFlag = 1;
}


/*! \brief Doze off for approx. 8 seconds (Vcc = 5 V)
 *
 * Waits for ADC-cycles to complete, disables the ADC, then sleeps for
 * approx. 8 seconds (Vcc = 5 V) using the watchdog timer.
 * On wakeup, ADC is re-enabled.
 * Modification by Martin Thomas:
 *  Do not enter standby mode if PB6 is low (MASTER_INT on BC100)
 *  so SPI communication with the master-controller is still possible
 *  during "doze".
 */
void Doze(void)
{
	uint8_t sreg_saved;
	// Wait for this ADC cycle to complete, then halt after the next one.
	ADC_Wait();
	ADCS.Halt = TRUE;
	ADCS.Flag = FALSE;

	do {
	} while (ADCS.Flag == FALSE);

	WDTCR = (1<<WDP3)|(1<<WDP0);            // 8.0 seconds at 5 volts VCC.
	WDTCR |= (1<<WDIF)|(1<<WDIE)|(1<<WDE);  // Clear flag and enable watchdog.

	DDRB  &= ~(1<<PB6);   // MASTER_INT as input
	PORTB |= (1<<PB6);    // enabled internal pullup
	WatchdogFlag = 0;
	if ( PINB & (1<<PB6) ) {
		MCUCR |= (1<<SE) | (1<<SM1) | (1<<SM0); // Sleep enable, mode = standby.
		__sleep();                              // Go to sleep, wake up by WDT.
	}
	else {
		// stay awake if PB6 is pulled low by master
		do {
		} while ( !(WatchdogFlag) );
	}

	__watchdog_reset();                     // Clear watchdog reset flag.
	PORTB &= ~(1<<PB6);    // disabled internal pullup
	MCUSR &= ~(1<<WDRF);

	sreg_saved = SREG;
	cli();
	WDTCR |= (1<<WDCE)|(1<<WDE);            // Watchdog change enable.
	WDTCR = 0;                              // Turn off watchdog.
	SREG = sreg_saved;

	ADCS.Halt = FALSE;                      // Enable consecutive runs of ADC.
	ADCSRA |= (1<<ADEN)|(1<<ADSC);          // Enable ADC & start conversion.

	// Wait for this cycle to complete.
	ADC_Wait();
}


/*! \brief Handles errors
 *
 * Stops PWM output and disables batteries. The function then goes into a loop
 * that starts with a call to Doze(), then attempts to handle each error. The
 * loop will reiterate until all flags are cleared.\n
 * The charger will reinitialize after this.
 *
 * Jumper errors are handled by clearing the flag, then calling JumperCheck().
 * If unsuccessful, the error flag will now have been set again.\n
 *
 * If there are no valid batteries, the loop will simply reiterate until a
 * valid battery is found. The error flag will then be cleared.\n
 *
 * In the case of PWM controller or battery temperature errors, the error
 * flag is simply cleared. This is because the problem may have gone away during
 * Doze(), or after reinitializing.\n
 *
 * If a battery is exhausted, we clear its exhausted-flag in \ref BattData,
 * and change batteries before clearing the error flag.
 *
 * \param inp Not used.
 */
unsigned char Error(unsigned char inp)
	{
	unsigned char i;

	PWM_Stop();           // Stop charging.
	DisableBatteries();   // Disable all loads.

	do {
		Doze();           // Take a nap.

		// For each bit in ErrorFlags, starting with LSB, handle
		// associated error, if the flag is set.
		for (i = 0x01; i!=0; i<<=1) {
			if(i & ErrorFlags) {
				switch (i) {

				case  ERR_JUMPER_MISMATCH:
					// Clear flag & recheck.
					ErrorFlags &= ~i;
					JumperCheck();
				break;


				case  ERR_NO_BATTERIES_ENABLED:
					// Clear if any battery gets enabled.
					if ((BattControl[0].Enabled) || (BattControl[1].Enabled)) {
							ErrorFlags &= ~i;
					}
				break;


				case  ERR_PWM_CONTROL:
					// Clear flag.
					ErrorFlags &= ~i;
				break;


				case  ERR_BATTERY_TEMPERATURE:
					// Clear flag.
					ErrorFlags &= ~i;
				break;


				case  ERR_BATTERY_EXHAUSTED:
					// Try the other battery.
					BattData.Exhausted = FALSE;
					BattActive = (BattActive + 1) % 2;
					ErrorFlags &= ~i;
				break;


				default:
				break;
				}
			}
		}
	} while (ErrorFlags);

	return(ST_INIT);
}


/*! \brief Sets the specified error-flag and saves the current state
 *
 * Updates \ref ErrorFlags and \ref ErrorState.
 *
 * \note Error flags are specified in statefunc.h.
 *
 * \param Flag Specifies what error to flag.
 */
void SetErrorFlag(unsigned char Flag)
{
	ErrorFlags |= Flag;
	ErrorState = CurrentState;
}


/*! \brief Checks on-board jumpers.
 *
 * Checks on-board jumpers by disconnecting all loads, engaging the PWM and
 * increasing the duty cycle until conditioned output voltage equals conditioned
 * input voltage. At low PWM duty and no load buck output should be zero and,
 * when increasing PWM duty, should quickly jump to steady state output roughly
 * equal to input voltage. Will disable and leave disabled all batteries.
 *
 * \retval FALSE If jumper or load mismatch.
 * \retval TRUE If everything OK.
 */
unsigned char JumperCheck(void)
{
	 DisableBatteries();       // Disconnect, or loads may be destroyed!

	 PWM_Start();              // Start PWM (controls the buck charger).

	 // Use general timer: shouldn't take longer than (6 x 255) / 2500 ~= 0.62s.
	 Time_Set(TIMER_GEN,0,1,0);

	do {
		// If the PWM output voltage saturates the ADC, stop PWM output and
		// report a failure.
		if (ADCS.rawVBAT == 1023) {
			PWM_Stop();
			return(FALSE);
		}

		// If the absolute difference between measured (VIN - VBAT) and the
		// typical value are below our set maximum, everything is OK.
		if (abs((signed int)(ADCS.VIN - VIN_VBAT_DIFF_TYP - ADCS.VBAT)) <
		                     VIN_VBAT_DIFF_MAX ) {

			PWM_Stop();
			return(TRUE);
		}

		// Charge current is too high -> check load and jumper J405 and J406.
		if (abs(ADCS.IBAT) > 100) {
			PWM_Stop();
			return(FALSE);
		}

		// If the PWM output can't be increased high enough -> check jumpers
		// J400-J404, J407 and J408.
		if (!PWM_IncrementDutyCycle()) {
			PWM_Stop();
			return(FALSE);
		}

      // Wait for ADC conversions to complete
		ADC_Wait();
	} while (Time_Left(TIMER_GEN));


	// If we end up here, the measurements took too long.
	PWM_Stop();
	return(FALSE);
}
